import GPUtil

from cral.tracking import log_params

from .models.classification import (VGG16, VGG19, Darknet53, DenseNet121,
                                    DenseNet169, DenseNet201, Detnet,
                                    EfficientNetB0, EfficientNetB1,
                                    EfficientNetB2, EfficientNetB3,
                                    EfficientNetB4, EfficientNetB5,
                                    EfficientNetB6, EfficientNetB7,
                                    InceptionResNetV2, InceptionV3, MobileNet,
                                    MobileNetV2, NASNetLarge, NASNetMobile,
                                    ResNet50, ResNet50V2, ResNet101,
                                    ResNet101V2, ResNet152, ResNet152V2,
                                    Xception, densely_connected_head)

classification_networks = dict(
    densenet121=DenseNet121,
    densenet169=DenseNet169,
    densenet201=DenseNet201,
    inceptionresnetv2=InceptionResNetV2,
    inceptionv3=InceptionV3,
    mobilenet=MobileNet,
    mobilenetv2=MobileNetV2,
    nasnetlarge=NASNetLarge,
    nasnetmobile=NASNetMobile,
    resnet50=ResNet50,
    resnet101=ResNet101,
    resnet152=ResNet152,
    resnet50v2=ResNet50V2,
    resnet101v2=ResNet101V2,
    resnet152v2=ResNet152V2,
    vgg16=VGG16,
    vgg19=VGG19,
    xception=Xception,
    efficientnetb0=EfficientNetB0,
    efficientnetb1=EfficientNetB1,
    efficientnetb2=EfficientNetB2,
    efficientnetb3=EfficientNetB3,
    efficientnetb4=EfficientNetB4,
    efficientnetb5=EfficientNetB5,
    efficientnetb6=EfficientNetB6,
    efficientnetb7=EfficientNetB7,
    darknet53=Darknet53,
    detnet=Detnet
    )

def log_gpu_params():

    gpu_data={}
    try:
        i=0;
        gpus= GPUtil.getGPUs()
        if len(gpus)>1:
            for gpu in gpus:
                gpu_data[f'GPU_{i}_Name']=gpu.name
                gpu_data[f'GPU_{i}_TotalMem']=gpu.memoryTotal
                i+=1
        else:
            for gpu in gpus:
                gpu_data[f'GPU_Name']=gpu.name
                gpu_data[f'GPU_TotalMem']=gpu.memoryTotal
    except Exception as e:
        e=e
    if len(gpu_data)>0:
        log_params(gpu_data)





